import bpy, bmesh, pprint

def segments_and_total_length(curve):
    """ Returns the length of all the segments of a curve and the total length"""
    
    origpoints = [p.co for s in curve.data.splines for p in s.bezier_points]
    
    bpy.context.scene.objects.active = curve
    curve.select = True
    bpy.ops.object.duplicate()
    obj = bpy.context.object
    obj.data.bevel_object = None
    bpy.ops.object.convert(keep_original=False)
    bm = bmesh.new()
    bm.from_mesh(obj.data)
    
    total = 0
    segs = []
    seglength = 0
    
    for e in bm.edges:
        l = e.calc_length()
        total += l
        seglength += l
        if e.verts[1].co in origpoints:
            segs.append(seglength)
            seglength = 0
            
    bpy.context.scene.objects.unlink(obj)
    
    return segs, total

def linearize(curve, duration):
    """ Calculates the 'virtual' F-Curve that needs to be added 
        to the F-Curve of the Bevel Factor of a curve."""
    
    segs, total = segments_and_total_length(curve)
    segcount = len(segs)
    
    
    
    at = 0
    points = {at:0}
    for i, seg in enumerate(segs):
        at += duration/total*seg
        #print("at="+str(duration)+"/"+str(total)+"*"+str(seg)+" = "+str(at))
        value = 1/segcount*(i+1)
        #print("value=1/"+str(segcount)+"*(1+"+str(i)+") = "+str(value))
        # linearDefault = value where a straight line would be
        linearDefault = 1/duration*at
        # we want to find the difference to the linearDefault
        points[at] = value - linearDefault
        
    return points

def absolute_linearize(curve, start, duration, min, max):
    """ linearized curve beginning at the specified start point"""
    rell = linearize(curve, duration)
    absl = {}
    for rkey in sorted(rell):
        v = min + (max-min)/duration*rkey + rell[rkey]*(max-min)
        #print (str(min)+"+"+str(max-min)+"/"+str(duration)+"*"+str(rkey)+"+"+str(rell[rkey])+"*"+str(max-min))
        absl[rkey+start] = min + (max-min)/duration*rkey + rell[rkey]*(max-min)
    #pprint.pprint(absl)
    return absl

def adjust_linear_fcurve(curve):
    """ with the remote control like abilities of python 
        this only works for linear animation fcurves. 
        Maybe we see a C-patch in the future, that deals with this 
        Blender internally"""
    orig_active = bpy.context.object
    bpy.ops.object.select_all(action='DESELECT')
    
    bpy.context.scene.objects.active = curve
    curve.select = True
    
    # getting start and end bevel factor animation fcurve
    fcurves = [fc for fc in curve.data.animation_data.action.fcurves if fc.data_path == "bevel_factor_end" or fc.data_path == "bevel_factor_start"]
    for fc in fcurves:
        pointlists = []
        # find ascending slope
        oldkf = None
        for kf in fc.keyframe_points:
            if oldkf == None:
                oldkf = kf
                continue
            if kf.co[1] > oldkf.co[1]:
                pointlists.append(absolute_linearize(curve, oldkf.co[0],kf.co[0]-oldkf.co[0], oldkf.co[1], kf.co[1]))
    
        for pl in pointlists:
            pk = [k for k in pl.keys()]
            pl.pop(pk[0])
            pl.pop(pk[-1])
            for at in sorted(pl):
                fc.keyframe_points.insert(at, pl[at])
                #pass
    bpy.context.scene.objects.active = orig_active
    orig_active.select = True
    
# in the console: 
# import linearCurveBevel as lb
# lb.adjust_linear_fcurve(C.object)