# mesh cleanup after import from e.g. 3ds files
# 
# select the objects you want to clean

import bpy
import mathutils

sce = bpy.context.scene

# make sure you don't have a camera selected
sel = [o for o in bpy.context.selected_objects if o.type == 'MESH']

# 1. remove double vertices
# 2. dissolve all aligned faces
for obj in sel:
    # activate object
    sce.objects.active = obj
    
    # remove all doubles
    bpy.ops.object.editmode_toggle()
    #bpy.ops.mesh.remove_doubles()
    #bpy.ops.mesh.select_all(action='DESELECT')
    bpy.ops.object.editmode_toggle()
    
    # dissolve all polygons with the same or opposite normal
    zero = mathutils.Vector()
    sameNormal = []
    sameNormals = []
    # [:] at the end makes it a copy
    polys = obj.data.polygons[:]
    
    while len(polys) > 0:
        # get the first poly
        p = polys[0]
        del polys[0]
        pn = mathutils.Vector(p.normal)
        # for the others test if their normal == p.normal (or opposite direction -> cross product)
        sameNormal = [other.index for other in polys if pn.cross(mathutils.Vector(other.normal)) == zero]
        
        # cannot switch to edit_mode since obj.data.polygons would be reallocated; 
        # doing this in a for-loop iterating over the polys would cause Blender to crash: 
        # http://www.blender.org/documentation/blender_python_api_2_62_1/info_gotcha.html#help-my-script-crashes-blender
        # therefore I store only the indeces
        
        polys[:] = [po for po in polys if po.index not in sameNormal]
        
        if len(sameNormal) > 0:
            sameNormal.append(p.index)
            sameNormals.append(sameNormal)
            sameNormal = []
        
    # put faces with the same normal into groups
    # (upon dissolve our indices would not be correct anymore)
    for sn in sameNormals:
        bpy.ops.object.editmode_toggle()
        bpy.ops.mesh.select_all(action='DESELECT')
        bpy.ops.object.editmode_toggle()
        for s in sn:
            obj.data.polygons[s].select = True
        bpy.ops.object.editmode_toggle()
        bpy.ops.object.vertex_group_assign(new=True)
        bpy.ops.object.editmode_toggle()
    
    # dissolve: select with vertex groups
    bpy.ops.object.editmode_toggle()
    bpy.ops.wm.context_set_value(data_path="tool_settings.mesh_select_mode", value="(False,False,True)")
    vg = obj.vertex_groups
    for i in range(len(vg)):
        vg.active_index = i
        bpy.ops.mesh.select_all(action='DESELECT') 
        bpy.ops.object.vertex_group_select()
        bpy.ops.mesh.dissolve()
        bpy.ops.mesh.select_all(action='DESELECT')

    # dissolve_limited: remove all unnecessary vertices
    for j in range(len(vg)):
        vg.active_index = j
        bpy.ops.mesh.select_all(action='DESELECT') 
        bpy.ops.object.vertex_group_select()
        bpy.ops.mesh.dissolve_limited()
        bpy.ops.mesh.select_all(action='DESELECT')
        
    for g in vg:
        vg.remove(g)
    
    bpy.ops.object.editmode_toggle()
    
    # set the origin to the geometry
    #bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')

